#!/usr/bin/env node
"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const i = __importStar(require("inquirer"));
const fs = __importStar(require("fs-extra"));
const path = __importStar(require("path"));
const config_api_1 = require("segment-typescript-api/cjs/config_api");
const Rx = __importStar(require("rxjs"));
const generate_protocol_1 = require("../generate_protocol");
const child_process_1 = require("child_process");
const TEMPLATE_DIR = path.join(__dirname, '..', '..', 'template');
const SAM_DIR = path.join(TEMPLATE_DIR, 'sam');
const JS_DIR = path.join(TEMPLATE_DIR, 'js');
const JS_PROJECT_DIR = path.join(TEMPLATE_DIR, 'js-project');
const TS_DIR = path.join(TEMPLATE_DIR, 'ts');
const TS_PROJECT_DIR = path.join(TEMPLATE_DIR, 'ts-project');
let cwd = process.cwd();
let answers = {
    fn_name: '',
    fn_type: 'source',
    language: 'javascript',
    work_slug: '',
    access_token: '',
    tp_name: ''
};
var prompts = new Rx.Subject();
i.prompt(prompts).ui.process.subscribe(answer => {
    let _answer = answer;
    answers[_answer.name] = _answer.answer;
    // answers = { ...answers, ...answer as Record<keyof typeof answers, any> }
    if (answers.tp_name) {
        prompts.complete();
    }
    else if (answers.work_slug && answers.access_token) {
        config_api_1.listAllTrackingPlans(answers.access_token, answers.work_slug).then(tps => {
            prompts.next({
                name: 'tp_name',
                message: 'Select tracking plan',
                type: 'list',
                choices: ['none', ...tps.tracking_plans.map(tps => tps.display_name)]
            });
        }).catch(e => {
            console.error('Error fetching tracking plans');
            console.error(e);
            prompts.complete();
        });
    }
}, error => {
    console.error(error);
}, () => {
    let p1 = fs.copyFile(path.join(SAM_DIR, 'package.json'), path.join(cwd, 'package.json')).then(() => {
        child_process_1.exec('npm i', (error, stdout) => {
            if (error) {
                console.error(error);
            }
        });
    });
    let p2 = fs.copyFile(path.join(SAM_DIR, 'template.yaml'), path.join(cwd, 'template.yaml'));
    let p3;
    let p4;
    let p5;
    let p6;
    if (answers.fn_type == 'source') {
        p3 = fs.copyFile(path.join(SAM_DIR, 'sam-src.js'), path.join(cwd, 'sam.js'));
        p5 = fs.copy(path.join(JS_DIR, 'tests', 'src-payload.json'), path.join(cwd, 'src', 'tests', 'src-payload.json'));
        if (answers.language == 'javascript') {
            p4 = fs.copy(path.join(JS_DIR, 'source.js'), path.join(cwd, 'src', 'function.js'));
            p6 = fs.copy(path.join(JS_DIR, 'tests', 'test-src-handler.js'), path.join(cwd, 'src', 'tests', 'test.js'));
        }
        else {
            p4 = fs.copy(path.join(TS_DIR, 'source.ts'), path.join(cwd, 'src', 'function.ts'));
            p6 = fs.copy(path.join(TS_DIR, 'tests', 'test-src-handler.ts'), path.join(cwd, 'src', 'tests', 'test.ts'));
        }
    }
    else {
        p3 = fs.copyFile(path.join(SAM_DIR, 'sam-dest.js'), path.join(cwd, 'sam.js'));
        p5 = fs.copy(path.join(JS_DIR, 'tests', 'dest-payload.json'), path.join(cwd, 'src', 'tests', 'dest-payload.json'));
        if (answers.language == 'javascript') {
            p4 = fs.copy(path.join(JS_DIR, 'destination.js'), path.join(cwd, 'src', 'function.js'));
            p6 = fs.copy(path.join(JS_DIR, 'tests', 'test-dest-handler.js'), path.join(cwd, 'src', 'tests', 'test.js'));
        }
        else {
            p4 = fs.copy(path.join(TS_DIR, 'destination.ts'), path.join(cwd, 'src', 'function.ts'));
            p6 = fs.copy(path.join(TS_DIR, 'tests', 'test-dest-handler.ts'), path.join(cwd, 'src', 'tests', 'test.ts'));
        }
    }
    let p7;
    if (answers.language == 'javascript') {
        p7 = fs.copy(JS_PROJECT_DIR, cwd);
    }
    else {
        p7 = fs.copy(TS_PROJECT_DIR, cwd);
    }
    if (answers.tp_name !== 'none') {
        generate_protocol_1.generate_protocol_definitions({
            access_token: answers.access_token,
            tp_name: answers.tp_name,
            work_slug: answers.work_slug,
            out_file: path.join(cwd, 'src', 'protocol.d.ts')
        });
    }
});
prompts.next({
    name: 'language',
    message: 'Select language',
    type: 'list',
    choices: ['javascript', 'typescript']
});
prompts.next({
    name: 'fn_type',
    message: 'Select function type',
    type: 'list',
    choices: ['source', 'destination']
});
prompts.next({
    name: 'fn_name',
    message: 'Select function name',
    type: 'input'
});
prompts.next({
    name: 'work_slug',
    message: 'Enter workspace slug',
    type: 'input'
});
prompts.next({
    name: 'access_token',
    message: 'Enter workspace access token (requires Function Admin permission)',
    type: 'input'
});
// listAllTrackingPlans()
// i.prompt([{
//   message: 'Select language',
//   choices: ['typescript', 'javascript']
// }, {
//   message: 'Select source function name (blank for none)',
//   type: 'input'
// }, {
//   message: 'Select destination function name (blank for none)',
//   type: 'input'
// }, {
//   message: 'Enter workspace slug',
//   type: 'input'
// }, {
//   message: 'Enter workspace access token (requires Function Admin permission)',
//   type: 'input'
// }]).then(answers => {
//   let [lang, src_name, dest_name, work_slug, access_token] = answers as string[]
//   let cwd = process.cwd()
// })
//# sourceMappingURL=init.js.map